From 306dc668352424c6398045effa394200fa34baa1 Mon Sep 17 00:00:00 2001
From: Shugo Maeda <shugo@ruby-lang.org>
Date: Wed, 11 Aug 2010 18:41:54 +0900
Subject: [PATCH 17/51] redefine optimized methods for classbox.

---
 lib/classbox.rb |   13 +++++++++++++
 1 files changed, 13 insertions(+), 0 deletions(-)

diff --git a/lib/classbox.rb b/lib/classbox.rb
index a5b1af8..91e9ab9 100644
--- a/lib/classbox.rb
+++ b/lib/classbox.rb
@@ -14,11 +14,24 @@ class Classbox < Module
     @__overlayed_modules__ = {}
   end
 
+  @@__optimized_methods__ = [
+    :+, :-, :*, :/, :%, :==, :===, :<, :<=, :<<, :[], :[]=, :>, :>=, :!, :!=,
+    :length, :size, :succ
+  ]
+
   def refine(klass, &block)
     mod = Module.new
     modules = @__overlayed_modules__[klass] ||= []
     modules.push(mod)
     overlay_module(klass, mod, binding(1))
+    mod.singleton_class.send(:define_method, :method_added) do |mid|
+      if @@__optimized_methods__.include?(mid)
+        klass.send(:alias_method, :__original_optimized_method__, mid)
+        klass.send(:define_method, mid) {}
+        klass.send(:alias_method, mid, :__original_optimized_method__)
+        klass.send(:remove_method, :__original_optimized_method__)
+      end
+    end
     mod.module_eval(&block)
   end
 end
-- 
1.7.0.4

